#include <math.h>
#include <stdio.h>
#include <cairo/cairo.h>
#include "counter.h"
#include "powers_table.h"

static cairo_surface_t *digits[10];

void counter_release(void) {
  for (int i = 0; i < 10; i++)
    if (digits[i])
      cairo_surface_destroy(digits[i]);
}

static cairo_surface_t *get_digit(int digit) {
  cairo_surface_t *surface;
  char buffer[16];

  surface = digits[digit];

  if (surface == NULL) {
    snprintf(buffer, sizeof(buffer), "./images/%d.png", digit);

    surface = digits[digit] = cairo_image_surface_create_from_png(buffer);
  }

  return surface;
}

static int count_digits(unsigned long number) {
  for (int i = 1; i < POWERS_COUNT; i++)
    if (number < powers_table[i][0]) return powers_table[i][1];

  return powers_table[POWERS_COUNT-1][1];
}

static int nth_digit(unsigned long number, int n) {
  return (number / powers_table[n][0]) % 10;
}

void counter_create(unsigned long number, char *filename) {
  int digits_count;
  cairo_surface_t *surface;
  cairo_t *cr;

  digits_count = count_digits(number);
  surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, digits_count * DIGIT_WIDTH, DIGIT_HEIGHT);
  cr = cairo_create(surface);
 
  for (int i = 0; i < digits_count; i++) {
    cairo_set_source_surface(cr, get_digit(nth_digit(number, digits_count-1-i)), i * DIGIT_WIDTH, 0);
    cairo_paint(cr);
  }

  cairo_surface_write_to_png(surface, filename);
  cairo_surface_destroy(surface);
  cairo_destroy(cr);
}
